package com.guo.task;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.guo.aly.ALYunEcsService;
import com.guo.aly.model.EcsSelectModel;
import com.guo.aly.model.InstanceInfo;
import com.guo.common.constant.Constans;
import com.guo.common.model.TotalNumber;
import com.guo.gateway.cache.*;
import com.guo.gateway.model.NodeInfo;
import com.guo.gateway.model.NodeJoin;
import com.guo.task.Contraction.ContractionVolume;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author gxb
 * @description 定时器去扫描节点情况
 * @date 2024-04-18 19:01
 */
@Component
@Log4j2
@AllArgsConstructor
public class Collection {

    /**
     * 最少节点数量
     */
    private final static Integer mainNumber = 1;

    /**
     * 判断连接阈值
     */
    private final static Integer maxNumber = 21;
    private final static Integer beforemaxNumber = 20;

    /**
     * 网关负载节点缓存
     */
    private final LoadNodeCache loadNodeCache;

    /**
     * 网关负载序列
     */
    private final LoadSeriesCache loadSeriesCache;

    /**
     * 网关节点缓存  存ip地址
     */
    private final NodeCache nodeCache;

    /**
     * 网关节点连接数
     */
    private final NodeScoreCache nodeScoreCache;

    /**
     * 网关节点存储VIN信息
     */
    private final NodeSetVinCache nodeSetVinCache;

    /**
     * 网关车辆对应网关节点
     */
    private final VehicleLineNodeCache vehicleLineNodeCache;

    /**
     * 缩容缓存
     */
    private final NodeReduced nodeReduced;

    /**
     * 扩缩容
     */
    @Autowired
    private ContractionVolume contractionVolume;

    /**
     * 阿里云方法
     */
    @Autowired
    private ALYunEcsService alYunEcsService;


    /**
     * 定时扫描节点信息
     */
    @Scheduled(cron = "0/10 * * * * ?")
    public void scheduledEcsCompanding() throws Exception {

        //查询阿里云是否存在实例
        EcsSelectModel ecsSelectModel = new EcsSelectModel();
        List<String> addArryList = new ArrayList<>();
        addArryList.add("Myname");
        ecsSelectModel.setInstanceNameList(addArryList);
        //实例集合
       List<InstanceInfo> instanceLists = alYunEcsService.selectECS(ecsSelectModel);

        //节点计数
        Long nodeNumber = Constans.NUMERICAL_VALUE_LONG_ZERO;

        //所有节点连接数总数
        long connectionTotal = Constans.NUMERICAL_VALUE_LONG_ZERO;

        //删除连接数缓存
        nodeScoreCache.delete();

       // 判断实例集合是否为空
        if (!instanceLists.isEmpty()) {

            //将实例存入缓存
            for (InstanceInfo instance : instanceLists) {

                //清空  缓存中节点信息
                nodeCache.remove(instance.getInstanceId());

                // 创建一个新的 NodeInfo 对象
                NodeInfo nodeInfo = new NodeInfo();
                // 设置实例的信息
                nodeInfo.setNodeId(instance.getInstanceId());
                nodeInfo.setPublicIdAddress(instance.getPublicIpAddress());
                nodeInfo.setPrivateIdAddress(instance.getPrivateIpAddress());
                // 将新的 NodeInfo 对象放入缓存
                nodeCache.put(nodeInfo);

                    //查询收集节点连接数
                    int connectSize = querymqttConnections(instance.getPublicIpAddress());

                    log.info("当前：" + instance.getInstanceId() + ",的连接数：" + connectSize);

                    //计数
                    nodeNumber++;
                    connectionTotal += connectSize;

                    //将连接数存入缓存
                    // key：网关负载业务 value:网关节点ID + 连接数
                    NodeJoin nodeJoin = new NodeJoin();
                    nodeJoin.setNodeId(instance.getInstanceId());
                    nodeJoin.setLinkingNumber(Long.valueOf(connectSize));
                    nodeScoreCache.save(nodeJoin);

                    //加层判断，把不满足缩容条件的缓存删除 连接数 > 最低阈值
                    if (connectSize > beforemaxNumber && nodeReduced.isWhether(instance.getInstanceId())) {
                        nodeReduced.remove(instance.getInstanceId());
                    }

                    //判断是否达到缩容条件  节点数量 > 1 或 连接数 < 21  必须满足节点数量在两个及以上，且连接数低于21
                    if (instanceLists.size() > mainNumber && connectSize < maxNumber) {
                        //调用缩容方法  记录
                        contractionVolume.reduction(instance.getInstanceId());
                    }
            }
        }

        //封装节点数量和节点连接总数
        TotalNumber totalNumber = new TotalNumber();
        totalNumber.setConnectionTotal(connectionTotal);
        totalNumber.setNodeNumber(nodeNumber);
        //调用扩容方法去判断是否需要扩容
        contractionVolume.contractionVolume(totalNumber);

    }


    public int  querymqttConnections(String publicIPaddress){

        //获取每个FluxMQ运行信息
        String URL = Constans.HTTP_REQUEST_HEAD
                + publicIPaddress
                + Constans.HTTP_REQUEST_ADDRESS;
        OkHttpClient client = new OkHttpClient();

        Request request = new Request.Builder()
                .url(URL)
                .get()
                .addHeader(Constans.HTTP_REQUEST_ADDHEADER_AGENT, Constans.HTTP_REQUEST_ADDHEADER_APIFOX)
                .addHeader(Constans.HTTP_REQUEST_ADDHEADER_ACCESSTOKEN, Constans.HTTP_REQUEST_NULL_STRING)
                .build();

        try {
            Response response = client.newCall(request).execute();
            JSONArray jsonArray = JSONArray.parseArray(response.body().string());
            JSONObject jsonObject = jsonArray.getJSONObject(Constans.NUMERICAL_VALUE_ZERO);
            JSONObject mqttInfo = jsonObject.getJSONObject(Constans.HTTP_REQUEST_MQTTINFO);
            return mqttInfo.getIntValue(Constans.HTTP_REQUEST_CONNECTSIZE);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
